@@ -4,17 +4,19 @@
 
 #include <clutter/x11/clutter-x11.h>
 
-#include "screen.h"
-#include "errors.h"
-#include "window.h"
+#include <meta/screen.h>
+#include <meta/errors.h>
+#include <meta/window.h>
 #include "compositor-private.h"
-#include "compositor-mutter.h"
+#include <meta/compositor-mutter.h>
 #include "xprops.h"
-#include "prefs.h"
-#include "mutter-window-private.h"
-#include "mutter-window-group.h"
-#include "../core/window-private.h" /* to check window->hidden */
-#include "../core/display-private.h" /* for meta_display_lookup_x_window() */
+#include <meta/prefs.h>
+#include <meta/meta-shadow-factory.h>
+#include "meta-window-actor-private.h"
+#include "meta-window-group.h"
+#include "meta-background-actor.h"
+#include "window-private.h" /* to check window->hidden */
+#include "display-private.h" /* for meta_display_lookup_x_window() */
 #include <X11/extensions/shape.h>
 #include <X11/extensions/Xcomposite.h>
 
@@ -33,28 +35,28 @@ composite_at_least_version (MetaDisplay *display, int maj, int min)
   return (major > maj || (major == maj && minor >= min));
 }
 
-static void sync_actor_stacking (GList *windows);
+static void sync_actor_stacking (MetaCompScreen *info);
 
 static void
-mutter_finish_workspace_switch (MetaCompScreen *info)
+meta_finish_workspace_switch (MetaCompScreen *info)
 {
   GList *l;
 
   /* Finish hiding and showing actors for the new workspace */
   for (l = info->windows; l; l = l->next)
-    mutter_window_sync_visibility (l->data);
+    meta_window_actor_sync_visibility (l->data);
 
   /*
    * Fix up stacking order in case the plugin messed it up.
    */
-  sync_actor_stacking (info->windows);
+  sync_actor_stacking (info);
 
 /*   printf ("... FINISHED DESKTOP SWITCH\n"); */
 
 }
 
 void
-mutter_switch_workspace_completed (MetaScreen *screen)
+meta_switch_workspace_completed (MetaScreen *screen)
 {
   MetaCompScreen *info = meta_screen_get_compositor_data (screen);
 
@@ -67,7 +69,7 @@ mutter_switch_workspace_completed (MetaScreen *screen)
     }
 
   if (!info->switch_workspace_in_progress)
-    mutter_finish_workspace_switch (info);
+    meta_finish_workspace_switch (info);
 }
 
 void
@@ -84,9 +86,9 @@ add_win (MetaWindow *window)
 
   g_return_if_fail (info != NULL);
 
-  mutter_window_new (window);
+  meta_window_actor_new (window);
 
-  sync_actor_stacking (info->windows);
+  sync_actor_stacking (info);
 }
 
 static void
@@ -94,16 +96,16 @@ process_damage (MetaCompositor     *compositor,
                 XDamageNotifyEvent *event,
                 MetaWindow         *window)
 {
-  MutterWindow *cw;
+  MetaWindowActor *window_actor;
 
   if (window == NULL)
     return;
 
-  cw = MUTTER_WINDOW (meta_window_get_compositor_private (window));
-  if (cw == NULL)
+  window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
+  if (window_actor == NULL)
     return;
 
-  mutter_window_process_damage (cw, event);
+  meta_window_actor_process_damage (window_actor, event);
 }
 
 #ifdef HAVE_SHAPE
@@ -112,18 +114,18 @@ process_shape (MetaCompositor *compositor,
                XShapeEvent    *event,
                MetaWindow     *window)
 {
-  MutterWindow *cw;
+  MetaWindowActor *window_actor;
 
   if (window == NULL)
     return;
 
-  cw = MUTTER_WINDOW (meta_window_get_compositor_private (window));
-  if (cw == NULL)
+  window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
+  if (window_actor == NULL)
     return;
 
   if (event->kind == ShapeBounding)
     {
-      mutter_window_update_shape (cw, event->shaped);
+      meta_window_actor_update_shape (window_actor, event->shaped);
     }
 }
 #endif
@@ -133,19 +135,35 @@ process_property_notify (MetaCompositor	*compositor,
                          XPropertyEvent *event,
                          MetaWindow     *window)
 {
-  MutterWindow *cw;
+  MetaWindowActor *window_actor;
+
+  if (event->atom == compositor->atom_x_root_pixmap)
+    {
+      GSList *l;
+
+      for (l = meta_display_get_screens (compositor->display); l; l = l->next)
+        {
+	  MetaScreen  *screen = l->data;
+          if (event->window == meta_screen_get_xroot (screen))
+            {
+              MetaCompScreen *info = meta_screen_get_compositor_data (screen);
+              meta_background_actor_update (META_BACKGROUND_ACTOR (info->background_actor));
+              return;
+            }
+        }
+    }
 
   if (window == NULL)
     return;
 
-  cw = MUTTER_WINDOW (meta_window_get_compositor_private (window));
-  if (cw == NULL)
+  window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
+  if (window_actor == NULL)
     return;
 
   /* Check for the opacity changing */
   if (event->atom == compositor->atom_net_wm_window_opacity)
     {
-      mutter_window_update_opacity (cw);
+      meta_window_actor_update_opacity (window_actor);
       DEBUG_TRACE ("process_property_notify: net_wm_window_opacity\n");
       return;
     }
@@ -185,13 +203,13 @@ get_output_window (MetaScreen *screen)
 }
 
 /**
- * mutter_get_stage_for_screen:
+ * meta_get_stage_for_screen:
  * @screen: a #MetaScreen
  *
  * Returns: (transfer none): The #ClutterStage for the screen
  */
 ClutterActor *
-mutter_get_stage_for_screen (MetaScreen *screen)
+meta_get_stage_for_screen (MetaScreen *screen)
 {
   MetaCompScreen *info = meta_screen_get_compositor_data (screen);
 
@@ -202,13 +220,13 @@ mutter_get_stage_for_screen (MetaScreen *screen)
 }
 
 /**
- * mutter_get_overlay_group_for_screen:
+ * meta_get_overlay_group_for_screen:
  * @screen: a #MetaScreen
  *
  * Returns: (transfer none): The overlay group corresponding to @screen
  */
 ClutterActor *
-mutter_get_overlay_group_for_screen (MetaScreen *screen)
+meta_get_overlay_group_for_screen (MetaScreen *screen)
 {
   MetaCompScreen *info = meta_screen_get_compositor_data (screen);
 
@@ -219,13 +237,13 @@ mutter_get_overlay_group_for_screen (MetaScreen *screen)
 }
 
 /**
- * mutter_get_window_group_for_screen:
+ * meta_get_window_group_for_screen:
  * @screen: a #MetaScreen
  *
  * Returns: (transfer none): The window group corresponding to @screen
  */
 ClutterActor *
-mutter_get_window_group_for_screen (MetaScreen *screen)
+meta_get_window_group_for_screen (MetaScreen *screen)
 {
   MetaCompScreen *info = meta_screen_get_compositor_data (screen);
 
@@ -236,13 +254,34 @@ mutter_get_window_group_for_screen (MetaScreen *screen)
 }
 
 /**
- * mutter_get_windows:
+ * meta_get_background_actor_for_screen:
  * @screen: a #MetaScreen
  *
- * Returns: (transfer none) (element-type Clutter.Actor): The windows on @screen
+ * Gets the actor that draws the root window background under the windows.
+ * The root window background automatically tracks the image or color set
+ * by the environment.
+ *
+ * Returns: (transfer none): The background actor corresponding to @screen
+ */
+ClutterActor *
+meta_get_background_actor_for_screen (MetaScreen *screen)
+{
+  MetaCompScreen *info = meta_screen_get_compositor_data (screen);
+
+  if (!info)
+    return NULL;
+
+  return info->background_actor;
+}
+
+/**
+ * meta_get_window_actors:
+ * @screen: a #MetaScreen
+ *
+ * Returns: (transfer none) (element-type Clutter.Actor): The set of #MetaWindowActor on @screen
  */
 GList *
-mutter_get_windows (MetaScreen *screen)
+meta_get_window_actors (MetaScreen *screen)
 {
   MetaCompScreen *info = meta_screen_get_compositor_data (screen);
 
@@ -253,7 +292,7 @@ mutter_get_windows (MetaScreen *screen)
 }
 
 static void
-do_set_stage_input_region (MetaScreen *screen,
+do_set_stage_input_region (MetaScreen   *screen,
                            XserverRegion region)
 {
   MetaCompScreen *info = meta_screen_get_compositor_data (screen);
@@ -266,8 +305,8 @@ do_set_stage_input_region (MetaScreen *screen,
 }
 
 void
-mutter_set_stage_input_region (MetaScreen *screen,
-                               XserverRegion region)
+meta_set_stage_input_region (MetaScreen   *screen,
+                             XserverRegion region)
 {
   MetaCompScreen *info = meta_screen_get_compositor_data (screen);
   MetaDisplay  *display = meta_screen_get_display (screen);
@@ -295,7 +334,7 @@ mutter_set_stage_input_region (MetaScreen *screen,
 }
 
 void
-mutter_empty_stage_input_region (MetaScreen *screen)
+meta_empty_stage_input_region (MetaScreen *screen)
 {
   /* Using a static region here is a bit hacky, but Metacity never opens more than
    * one XDisplay, so it works fine. */
@@ -308,16 +347,16 @@ mutter_empty_stage_input_region (MetaScreen *screen)
       region = XFixesCreateRegion (xdpy, NULL, 0);
     }
 
-  mutter_set_stage_input_region (screen, region);
+  meta_set_stage_input_region (screen, region);
 }
 
 gboolean
-mutter_begin_modal_for_plugin (MetaScreen       *screen,
-                               MutterPlugin     *plugin,
-                               Window            grab_window,
-                               Cursor            cursor,
-                               MetaModalOptions  options,
-                               guint32           timestamp)
+meta_begin_modal_for_plugin (MetaScreen       *screen,
+                             MetaPlugin       *plugin,
+                             Window            grab_window,
+                             Cursor            cursor,
+                             MetaModalOptions  options,
+                             guint32           timestamp)
 {
   /* To some extent this duplicates code in meta_display_begin_grab_op(), but there
    * are significant differences in how we handle grabs that make it difficult to
@@ -382,9 +421,9 @@ mutter_begin_modal_for_plugin (MetaScreen       *screen,
 }
 
 void
-mutter_end_modal_for_plugin (MetaScreen       *screen,
-                             MutterPlugin     *plugin,
-                             guint32           timestamp)
+meta_end_modal_for_plugin (MetaScreen     *screen,
+                           MetaPlugin     *plugin,
+                           guint32         timestamp)
 {
   MetaDisplay    *display    = meta_screen_get_display (screen);
   Display        *xdpy = meta_display_get_xdisplay (display);
@@ -408,15 +447,15 @@ mutter_end_modal_for_plugin (MetaScreen       *screen,
  * a left-over modal grab for this screen.
  */
 void
-mutter_check_end_modal (MetaScreen *screen)
+meta_check_end_modal (MetaScreen *screen)
 {
   MetaDisplay    *display    = meta_screen_get_display (screen);
   MetaCompositor *compositor = display->compositor;
 
   if (compositor->modal_plugin &&
-      mutter_plugin_get_screen (compositor->modal_plugin) == screen)
+      meta_plugin_get_screen (compositor->modal_plugin) == screen)
     {
-      mutter_end_modal_for_plugin (screen,
+      meta_end_modal_for_plugin (screen,
                                    compositor->modal_plugin,
                                    CurrentTime);
     }
@@ -444,7 +483,7 @@ meta_compositor_manage_screen (MetaCompositor *compositor,
   XCompositeRedirectSubwindows (xdisplay, xroot, CompositeRedirectManual);
   XSync (xdisplay, FALSE);
 
-  if (meta_error_trap_pop_with_return (display, FALSE))
+  if (meta_error_trap_pop_with_return (display))
     {
       g_warning ("Another compositing manager is running on screen %i",
                  screen_number);
@@ -492,10 +531,15 @@ meta_compositor_manage_screen (MetaCompositor *compositor,
 
   XSelectInput (xdisplay, xwin, event_mask);
 
-  info->window_group = mutter_window_group_new (screen);
+  info->window_group = meta_window_group_new (screen);
+  info->background_actor = meta_background_actor_new (screen);
   info->overlay_group = clutter_group_new ();
   info->hidden_group = clutter_group_new ();
 
+  clutter_container_add (CLUTTER_CONTAINER (info->window_group),
+                         info->background_actor,
+                         NULL);
+
   clutter_container_add (CLUTTER_CONTAINER (info->stage),
                          info->window_group,
                          info->overlay_group,
@@ -505,19 +549,8 @@ meta_compositor_manage_screen (MetaCompositor *compositor,
   clutter_actor_hide (info->hidden_group);
 
   info->plugin_mgr =
-    mutter_plugin_manager_get (screen);
-
-  if (info->plugin_mgr != mutter_plugin_manager_get_default ())
-    {
-      /* The default plugin manager has been initialized during
-       * global preferences load.
-       */
-      if (!mutter_plugin_manager_load (info->plugin_mgr))
-        g_critical ("failed to load plugins");
-    }
-
-  if (!mutter_plugin_manager_initialize (info->plugin_mgr))
-    g_critical ("failed to initialize plugins");
+    meta_plugin_manager_get (screen);
+  meta_plugin_manager_initialize (info->plugin_mgr);
 
   /*
    * Delay the creation of the overlay window as long as we can, to avoid
@@ -568,21 +601,21 @@ meta_compositor_add_window (MetaCompositor    *compositor,
 
   add_win (window);
 
-  meta_error_trap_pop (display, FALSE);
+  meta_error_trap_pop (display);
 }
 
 void
 meta_compositor_remove_window (MetaCompositor *compositor,
                                MetaWindow     *window)
 {
-  MutterWindow         *cw     = NULL;
+  MetaWindowActor         *window_actor     = NULL;
 
   DEBUG_TRACE ("meta_compositor_remove_window\n");
-  cw = MUTTER_WINDOW (meta_window_get_compositor_private (window));
-  if (!cw)
+  window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
+  if (!window_actor)
     return;
 
-  mutter_window_destroy (cw);
+  meta_window_actor_destroy (window_actor);
 }
 
 void
@@ -620,7 +653,7 @@ meta_compositor_process_event (MetaCompositor *compositor,
 {
   if (compositor->modal_plugin && is_grabbed_event (event))
     {
-      MutterPluginClass *klass = MUTTER_PLUGIN_GET_CLASS (compositor->modal_plugin);
+      MetaPluginClass *klass = META_PLUGIN_GET_CLASS (compositor->modal_plugin);
 
       if (klass->xevent_filter)
         klass->xevent_filter (compositor->modal_plugin, event);
@@ -638,7 +671,7 @@ meta_compositor_process_event (MetaCompositor *compositor,
       screen = meta_window_get_screen (window);
       info = meta_screen_get_compositor_data (screen);
 
-      if (mutter_plugin_manager_xevent_filter (info->plugin_mgr, event))
+      if (meta_plugin_manager_xevent_filter (info->plugin_mgr, event))
 	{
 	  DEBUG_TRACE ("meta_compositor_process_event (filtered,window==NULL)\n");
 	  return TRUE;
@@ -657,7 +690,7 @@ meta_compositor_process_event (MetaCompositor *compositor,
 
 	  info = meta_screen_get_compositor_data (screen);
 
-	  if (mutter_plugin_manager_xevent_filter (info->plugin_mgr, event))
+	  if (meta_plugin_manager_xevent_filter (info->plugin_mgr, event))
 	    {
 	      DEBUG_TRACE ("meta_compositor_process_event (filtered,window==NULL)\n");
 	      return TRUE;
@@ -715,12 +748,12 @@ meta_compositor_show_window (MetaCompositor *compositor,
 			     MetaWindow	    *window,
                              MetaCompEffect  effect)
 {
-  MutterWindow *cw = MUTTER_WINDOW (meta_window_get_compositor_private (window));
+  MetaWindowActor *window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
   DEBUG_TRACE ("meta_compositor_show_window\n");
-  if (!cw)
+  if (!window_actor)
     return;
 
-  mutter_window_show (cw, effect);
+  meta_window_actor_show (window_actor, effect);
 }
 
 void
@@ -728,12 +761,12 @@ meta_compositor_hide_window (MetaCompositor *compositor,
                              MetaWindow     *window,
                              MetaCompEffect  effect)
 {
-  MutterWindow *cw = MUTTER_WINDOW (meta_window_get_compositor_private (window));
+  MetaWindowActor *window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
   DEBUG_TRACE ("meta_compositor_hide_window\n");
-  if (!cw)
+  if (!window_actor)
     return;
 
-  mutter_window_hide (cw, effect);
+  meta_window_actor_hide (window_actor, effect);
 }
 
 void
@@ -742,12 +775,12 @@ meta_compositor_maximize_window (MetaCompositor    *compositor,
 				 MetaRectangle	   *old_rect,
 				 MetaRectangle	   *new_rect)
 {
-  MutterWindow	 *cw = MUTTER_WINDOW (meta_window_get_compositor_private (window));
+  MetaWindowActor *window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
   DEBUG_TRACE ("meta_compositor_maximize_window\n");
-  if (!cw)
+  if (!window_actor)
     return;
 
-  mutter_window_maximize (cw, old_rect, new_rect);
+  meta_window_actor_maximize (window_actor, old_rect, new_rect);
 }
 
 void
@@ -756,12 +789,12 @@ meta_compositor_unmaximize_window (MetaCompositor    *compositor,
 				   MetaRectangle     *old_rect,
 				   MetaRectangle     *new_rect)
 {
-  MutterWindow	 *cw = MUTTER_WINDOW (meta_window_get_compositor_private (window));
+  MetaWindowActor *window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
   DEBUG_TRACE ("meta_compositor_unmaximize_window\n");
-  if (!cw)
+  if (!window_actor)
     return;
 
-  mutter_window_unmaximize (cw, old_rect, new_rect);
+  meta_window_actor_unmaximize (window_actor, old_rect, new_rect);
 }
 
 void
@@ -774,7 +807,7 @@ meta_compositor_update_workspace_geometry (MetaCompositor *compositor,
    */
   MetaScreen     *screen = meta_workspace_get_screen (workspace);
   MetaCompScreen *info;
-  MutterPluginManager *mgr;
+  MetaPluginManager *mgr;
 
   DEBUG_TRACE ("meta_compositor_update_workspace_geometry\n");
   info = meta_screen_get_compositor_data (screen);
@@ -783,7 +816,7 @@ meta_compositor_update_workspace_geometry (MetaCompositor *compositor,
   if (!mgr || !workspace)
     return;
 
-  mutter_plugin_manager_update_workspace (mgr, workspace);
+  meta_plugin_manager_update_workspace (mgr, workspace);
 #endif
 }
 
@@ -809,7 +842,7 @@ meta_compositor_switch_workspace (MetaCompositor     *compositor,
   info->switch_workspace_in_progress++;
 
   if (!info->plugin_mgr ||
-      !mutter_plugin_manager_switch_workspace (info->plugin_mgr,
+      !meta_plugin_manager_switch_workspace (info->plugin_mgr,
 					       from_indx,
 					       to_indx,
 					       direction))
@@ -821,23 +854,78 @@ meta_compositor_switch_workspace (MetaCompositor     *compositor,
        * necessarily change during the window hiding/unhiding, only their
        * relative position toward the destkop window.
        */
-      mutter_finish_workspace_switch (info);
+      meta_finish_workspace_switch (info);
     }
 }
 
 static void
-sync_actor_stacking (GList *windows)
+sync_actor_stacking (MetaCompScreen *info)
 {
+  GList *children;
   GList *tmp;
+  GList *old;
+  gboolean reordered;
 
-  /* NB: The first entry in the list is stacked the lowest */
+  /* NB: The first entries in the lists are stacked the lowest */
 
-  for (tmp = g_list_last (windows); tmp != NULL; tmp = tmp->prev)
+  /* Restacking will trigger full screen redraws, so it's worth a
+   * little effort to make sure we actually need to restack before
+   * we go ahead and do it */
+
+  children = clutter_container_get_children (CLUTTER_CONTAINER (info->window_group));
+  reordered = FALSE;
+
+  old = children;
+
+  /* We allow for actors in the window group other than the actors we
+   * know about, but it's up to a plugin to try and keep them stacked correctly
+   * (we really need extra API to make that reliable.)
+   */
+
+  /* Of the actors we know, the bottom actor should be the background actor */
+
+  while (old && old->data != info->background_actor && !META_IS_WINDOW_ACTOR (old->data))
+    old = old->next;
+  if (old == NULL || old->data != info->background_actor)
     {
-      MutterWindow *cw = tmp->data;
-
-      clutter_actor_lower_bottom (CLUTTER_ACTOR (cw));
+      reordered = TRUE;
+      goto done_with_check;
     }
+
+  /* Then the window actors should follow in sequence */
+
+  old = old->next;
+  for (tmp = info->windows; tmp != NULL; tmp = tmp->next)
+    {
+      while (old && !META_IS_WINDOW_ACTOR (old->data))
+        old = old->next;
+
+      /* old == NULL: someone reparented a window out of the window group,
+       * order undefined, always restack */
+      if (old == NULL || old->data != tmp->data)
+        {
+          reordered = TRUE;
+          goto done_with_check;
+        }
+
+      old = old->next;
+    }
+
+ done_with_check:
+
+  g_list_free (children);
+
+  if (!reordered)
+    return;
+
+  for (tmp = g_list_last (info->windows); tmp != NULL; tmp = tmp->prev)
+    {
+      MetaWindowActor *window_actor = tmp->data;
+
+      clutter_actor_lower_bottom (CLUTTER_ACTOR (window_actor));
+    }
+
+  clutter_actor_lower_bottom (info->background_actor);
 }
 
 void
@@ -858,12 +946,12 @@ meta_compositor_sync_stack (MetaCompositor  *compositor,
 
   /* Sources: first window is the highest */
   stack = g_list_copy (stack); /* The new stack of MetaWindow */
-  old_stack = g_list_reverse (info->windows); /* The old stack of MutterWindow */
+  old_stack = g_list_reverse (info->windows); /* The old stack of MetaWindowActor */
   info->windows = NULL;
 
   while (TRUE)
     {
-      MutterWindow *old_actor = NULL, *stack_actor = NULL, *actor;
+      MetaWindowActor *old_actor = NULL, *stack_actor = NULL, *actor;
       MetaWindow *old_window = NULL, *stack_window = NULL, *window;
 
       /* Find the remaining top actor in our existing stack (ignoring
@@ -871,11 +959,14 @@ meta_compositor_sync_stack (MetaCompositor  *compositor,
       while (old_stack)
         {
           old_actor = old_stack->data;
-          old_window = mutter_window_get_meta_window (old_actor);
+          old_window = meta_window_actor_get_meta_window (old_actor);
 
           if (old_window->hidden &&
-              !mutter_window_effect_in_progress (old_actor))
-            old_stack = g_list_delete_link (old_stack, old_stack);
+              !meta_window_actor_effect_in_progress (old_actor))
+            {
+              old_stack = g_list_delete_link (old_stack, old_stack);
+              old_actor = NULL;
+            }
           else
             break;
         }
@@ -884,10 +975,10 @@ meta_compositor_sync_stack (MetaCompositor  *compositor,
       while (stack)
         {
           stack_window = stack->data;
-          stack_actor = MUTTER_WINDOW (meta_window_get_compositor_private (stack_window));
+          stack_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (stack_window));
           if (!stack_actor)
             {
-              meta_verbose ("Failed to find corresponding MutterWindow "
+              meta_verbose ("Failed to find corresponding MetaWindowActor "
                             "for window %s\n", meta_window_get_description (stack_window));
               stack = g_list_delete_link (stack, stack);
             }
@@ -926,48 +1017,48 @@ meta_compositor_sync_stack (MetaCompositor  *compositor,
       old_stack = g_list_remove (old_stack, actor);
     }
 
-  sync_actor_stacking (info->windows);
+  sync_actor_stacking (info);
 }
 
 void
 meta_compositor_window_mapped (MetaCompositor *compositor,
                                MetaWindow     *window)
 {
-  MutterWindow *cw = MUTTER_WINDOW (meta_window_get_compositor_private (window));
+  MetaWindowActor *window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
   DEBUG_TRACE ("meta_compositor_window_mapped\n");
-  if (!cw)
+  if (!window_actor)
     return;
 
-  mutter_window_mapped (cw);
+  meta_window_actor_mapped (window_actor);
 }
 
 void
 meta_compositor_window_unmapped (MetaCompositor *compositor,
                                  MetaWindow     *window)
 {
-  MutterWindow *cw = MUTTER_WINDOW (meta_window_get_compositor_private (window));
+  MetaWindowActor *window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
   DEBUG_TRACE ("meta_compositor_window_unmapped\n");
-  if (!cw)
+  if (!window_actor)
     return;
 
-  mutter_window_unmapped (cw);
+  meta_window_actor_unmapped (window_actor);
 }
 
 void
 meta_compositor_sync_window_geometry (MetaCompositor *compositor,
 				      MetaWindow *window)
 {
-  MutterWindow	 *cw = MUTTER_WINDOW (meta_window_get_compositor_private (window));
-  MetaScreen	 *screen = meta_window_get_screen (window);
-  MetaCompScreen *info = meta_screen_get_compositor_data (screen);
+  MetaWindowActor *window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
+  MetaScreen      *screen = meta_window_get_screen (window);
+  MetaCompScreen  *info = meta_screen_get_compositor_data (screen);
 
   DEBUG_TRACE ("meta_compositor_sync_window_geometry\n");
   g_return_if_fail (info);
 
-  if (!cw)
+  if (!window_actor)
     return;
 
-  mutter_window_sync_actor_position (cw);
+  meta_window_actor_sync_actor_position (window_actor);
 }
 
 void
@@ -983,6 +1074,8 @@ meta_compositor_sync_screen_size (MetaCompositor  *compositor,
 
   clutter_actor_set_size (info->stage, width, height);
 
+  meta_background_actor_screen_size_changed (META_BACKGROUND_ACTOR (info->background_actor));
+
   meta_verbose ("Changed size for stage on screen %d to %dx%d\n",
 		meta_screen_get_screen_number (screen),
 		width, height);
@@ -994,11 +1087,11 @@ pre_paint_windows (MetaCompScreen *info)
   GList *l;
 
   for (l = info->windows; l; l = l->next)
-    mutter_window_pre_paint (l->data);
+    meta_window_actor_pre_paint (l->data);
 }
 
 static gboolean
-mutter_repaint_func (gpointer data)
+meta_repaint_func (gpointer data)
 {
   MetaCompositor *compositor = data;
   GSList *screens = meta_display_get_screens (compositor->display);
@@ -1017,6 +1110,26 @@ mutter_repaint_func (gpointer data)
   return TRUE;
 }
 
+static void
+on_shadow_factory_changed (MetaShadowFactory *factory,
+                           MetaCompositor    *compositor)
+{
+  GSList *screens = meta_display_get_screens (compositor->display);
+  GList *l;
+  GSList *sl;
+
+  for (sl = screens; sl; sl = sl->next)
+    {
+      MetaScreen *screen = sl->data;
+      MetaCompScreen *info = meta_screen_get_compositor_data (screen);
+      if (!info)
+        continue;
+
+      for (l = info->windows; l; l = l->next)
+        meta_window_actor_invalidate_shadow (l->data);
+    }
+}
+
 /**
  * meta_compositor_new: (skip)
  *
@@ -1040,18 +1153,23 @@ meta_compositor_new (MetaDisplay *display)
 
   compositor->display = display;
 
-  if (g_getenv("MUTTER_DISABLE_MIPMAPS"))
+  if (g_getenv("META_DISABLE_MIPMAPS"))
     compositor->no_mipmaps = TRUE;
 
   meta_verbose ("Creating %d atoms\n", (int) G_N_ELEMENTS (atom_names));
   XInternAtoms (xdisplay, atom_names, G_N_ELEMENTS (atom_names),
                 False, atoms);
 
+  g_signal_connect (meta_shadow_factory_get_default (),
+                    "changed",
+                    G_CALLBACK (on_shadow_factory_changed),
+                    compositor);
+
   compositor->atom_x_root_pixmap = atoms[0];
   compositor->atom_x_set_root = atoms[1];
   compositor->atom_net_wm_window_opacity = atoms[2];
 
-  compositor->repaint_func_id = clutter_threads_add_repaint_func (mutter_repaint_func,
+  compositor->repaint_func_id = clutter_threads_add_repaint_func (meta_repaint_func,
                                                                   compositor,
                                                                   NULL);
 
@@ -1059,11 +1177,11 @@ meta_compositor_new (MetaDisplay *display)
 }
 
 /**
- * mutter_get_overlay_window: (skip)
+ * meta_get_overlay_window: (skip)
  *
  */
 Window
-mutter_get_overlay_window (MetaScreen *screen)
+meta_get_overlay_window (MetaScreen *screen)
 {
   MetaCompScreen *info = meta_screen_get_compositor_data (screen);
 
