@@ -24,7 +24,7 @@
 #include <config.h>
 #include "iconcache.h"
 #include "ui.h"
-#include "errors.h"
+#include <meta/errors.h>
 
 #include <X11/Xatom.h>
 
@@ -241,7 +241,7 @@ read_rgb_icon (MetaDisplay   *display,
 			       0, G_MAXLONG,
 			       False, XA_CARDINAL, &type, &format, &nitems,
 			       &bytes_after, &data);
-  err = meta_error_trap_pop_with_return (display, TRUE);
+  err = meta_error_trap_pop_with_return (display);
 
   if (err != Success ||
       result != Success)
@@ -394,21 +394,19 @@ try_pixmap_and_mask (MetaDisplay *display,
 
   get_pixmap_geometry (display, src_pixmap, &w, &h, NULL);
 
-  unscaled = meta_gdk_pixbuf_get_from_pixmap (NULL,
-                                              src_pixmap,
-                                              0, 0, 0, 0,
+  unscaled = meta_gdk_pixbuf_get_from_pixmap (src_pixmap,
+                                              0, 0,
                                               w, h);
 
   if (unscaled && src_mask != None)
     {
       get_pixmap_geometry (display, src_mask, &w, &h, NULL);
-      mask = meta_gdk_pixbuf_get_from_pixmap (NULL,
-                                              src_mask,
-                                              0, 0, 0, 0,
+      mask = meta_gdk_pixbuf_get_from_pixmap (src_mask,
+                                              0, 0,
                                               w, h);
     }
 
-  meta_error_trap_pop (display, FALSE);
+  meta_error_trap_pop (display);
 
   if (mask)
     {
@@ -484,7 +482,7 @@ get_kwm_win_icon (MetaDisplay *display,
 			       &bytes_after, &data);
   icons = (Pixmap *)data;
 
-  err = meta_error_trap_pop_with_return (display, TRUE);
+  err = meta_error_trap_pop_with_return (display);
   if (err != Success ||
       result != Success)
     return;
