@@ -26,9 +26,9 @@
 #define _POSIX_C_SOURCE 200112L /* for fdopen() */
 
 #include <config.h>
-#include "common.h"
-#include "util.h"
-#include "main.h"
+#include <meta/common.h>
+#include <meta/util.h>
+#include <meta/main.h>
 
 #include <clutter/clutter.h> /* For clutter_threads_add_repaint_func() */
 
@@ -40,11 +40,12 @@
 #include <X11/Xlib.h>   /* must explicitly be included for Solaris; #326746 */
 #include <X11/Xutil.h>  /* Just for the definition of the various gravities */
 
+#ifdef WITH_VERBOSE_MODE
 static void
 meta_topic_real_valist (MetaDebugTopic topic,
                         const char    *format,
                         va_list        args);
-
+#endif
 
 #ifdef HAVE_BACKTRACE
 #include <execinfo.h>
