@@ -40,10 +40,10 @@
 
 #include <config.h>
 #include "window-props.h"
-#include "errors.h"
+#include <meta/errors.h>
 #include "xprops.h"
-#include "frame-private.h"
-#include "group.h"
+#include "frame.h"
+#include <meta/group.h>
 #include <X11/Xatom.h>
 #include <unistd.h>
 #include <string.h>
@@ -442,7 +442,7 @@ set_title_text (MetaWindow  *window,
       XDeleteProperty (window->display->xdisplay,
                        window->xwindow,
                        atom);
-      meta_error_trap_pop (window->display, FALSE);
+      meta_error_trap_pop (window->display);
     }
 
   return modified;
